@@ -4,7 +4,7 @@
  *
  *   Purpose:	Implements class interface for netCDF over C interface
  *
- *   $Header: /upc/share/CVS/netcdf/cxx/netcdf.cc,v 1.70 1996/07/02 16:03:15 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cc,v 1.1 1997/04/21 15:44:55 russ Exp $
  *********************************************************************/
 
 #include <string.h>
@@ -294,17 +294,23 @@ int NcFile::id( void ) const
     return the_id;
 }
 
-NcFile::NcFile( const char * path, FileMode mode )
+NcFile::NcFile( const char * path, FileMode fmode )
 {
     NcError err(NcError::silent_nonfatal); // constructor must not fail
 
-    switch (mode) {
-    case ReadOnly:
+    int mode = NC_NOWRITE;
+
+    switch (fmode) {
     case Write:
+	mode = NC_WRITE;
+	/*FALLTHRU*/
+    case ReadOnly:
 	the_id = ncopen(path, mode);
 	in_define_mode = 0;
 	break;
     case New:
+	mode = NC_NOCLOBBER;
+	/*FALLTHRU*/
     case Replace:
 	the_id = nccreate(path, mode);
 	in_define_mode = 1;
