@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, University Corporation for Atmospheric Research
  *   See netcdf/README file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf/ncdump/ncdump.c,v 1.50 1996/05/14 16:45:14 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncdump/ncdump.c,v 1.6 1997/05/19 20:49:03 russ Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -15,25 +15,25 @@
 #include "dumplib.h"
 #include "vardata.h"
 
-#ifndef NO_HAVE_PROTOTYPES 
 static void usage(void);
-static char* name_path(char* path);
+static char* name_path(const char* path);
 static char* type_name(nc_type  type);
 static void tztrim(char* ss);
-static void pr_att_vals(nc_type  type, int len, void* vals);
-static void do_ncdump(char* path, struct fspec* specp);
+static void pr_att_string(size_t len, const char* string);
+static void pr_att_vals(nc_type  type, size_t len, const double* vals);
+static void pr_att(int ncid, int varid, const char *varname, int ia);
+static void do_ncdump(const char* path, struct fspec* specp);
 static void make_lvars(char* optarg, struct fspec* fspecp);
-static void set_sigdigs(char* optarg);
-static void set_precision(char* optarg);
+static void set_sigdigs( const char* optarg);
+static void set_precision( const char *optarg);
 int main(int argc, char** argv);
-#endif
 
 #define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)
 
 char *progname;
 
 static void
-usage()
+usage(void)
 {
 #define USAGE   "\
   [-c]             Coordinate variable data and header information\n\
@@ -43,7 +43,7 @@ usage()
   [-f [c|f]]       Full annotations for C or Fortran indices in data\n\
   [-l len]         Line length maximum in data section (default 80)\n\
   [-n name]        Name for netCDF (default derived from file name)\n\
-  [-p n[,n]]       Display float (double) values with specified precision\n\
+  [-p n[,n]]       Display floating-point values with less precision\n\
   file             File name of input netCDF file\n"
 
     (void) fprintf(stderr,
@@ -59,10 +59,11 @@ usage()
  * last component of path and stripping off any extension.
  */
 static char *
-name_path(path)
-     char *path;
+name_path(const char *path)
 {
-    char *cp, *new;
+    const char *cp;
+    char *new;
+    char *sp;
 
 #ifdef vms
 #define FILE_DELIMITER ']'
@@ -81,18 +82,16 @@ name_path(path)
     new = (char *) malloc((unsigned) (strlen(cp)+1));
     if (new == 0) {
 	error("out of memory!");
-	exit(EXIT_FAILURE);
     }
     (void) strcpy(new, cp);	/* copy last component of path */
-    if ((cp = strrchr(new, '.')) != NULL)
-      *cp = '\0';		/* strip off any extension */
+    if ((sp = strrchr(new, '.')) != NULL)
+      *sp = '\0';		/* strip off any extension */
     return new;
 }
 
 
 static char *
-type_name(type)
-     nc_type type;
+type_name(nc_type type)
 {
     switch (type) {
       case NC_BYTE:
@@ -101,8 +100,8 @@ type_name(type)
 	return "char";
       case NC_SHORT:
 	return "short";
-      case NC_LONG:
-	return "long";
+      case NC_INT:
+	return "int";
       case NC_FLOAT:
 	return "float";
       case NC_DOUBLE:
@@ -120,8 +119,7 @@ type_name(type)
  * might include an exponent part.
  */
 static void
-tztrim(ss)
-     char *ss;			/* returned string representing dd */
+tztrim(char *ss)
 {
     char *cp, *ep;
     
@@ -146,118 +144,202 @@ tztrim(ss)
 
 
 /*
- * Print list of attribute values.  Attribute values must be printed with
- * explicit type tags, because their types are not declared.
+ * Print attribute string, for text attributes.
  */
 static void
-pr_att_vals(type, len, vals)
-     nc_type type;
-     int len;
-     void *vals;
+pr_att_string(
+     size_t len,
+     const char *string
+     )
 {
     int iel;
-    union {
-	char *cp;
-	short *sp;
-	nclong *lp;
-	float *fp;
-	double *dp;
-    } gp;
-    char *sp;
+    const char *cp;
+    const char *sp;
     unsigned char uc;
-    char gps[30];		/* for ascii of a float or double precision */
 
+    cp = string;
+    Printf ("\"");
+    /* adjust len so trailing nulls don't get printed */
+    sp = cp + len - 1;
+    while (len != 0 && *sp-- == '\0')
+	len--;
+    for (iel = 0; iel < len; iel++)
+	switch (uc = *cp++ & 0377) {
+	case '\b':
+	    Printf ("\\b");
+	    break;
+	case '\f':
+	    Printf ("\\f");
+	    break;
+	case '\n':		/* generate linebreaks after new-lines */
+	    Printf ("\\n\",\n    \"");
+	    break;
+	case '\r':
+	    Printf ("\\r");
+	    break;
+	case '\t':
+	    Printf ("\\t");
+	    break;
+	case '\v':
+	    Printf ("\\v");
+	    break;
+	case '\\':
+	    Printf ("\\\\");
+	    break;
+	case '\'':
+	    Printf ("\\'");
+	    break;
+	case '\"':
+	    Printf ("\\\"");
+	    break;
+	default:
+	    Printf ("%c",uc);
+	    break;
+	}
+    Printf ("\"");
+
+}
+
+
+/*
+ * Print list of attribute values, for numeric attributes.  Attribute values
+ * must be printed with explicit type tags, because CDL doesn't have explicit
+ * syntax to declare an attribute type.
+ */
+static void
+pr_att_vals(
+     nc_type type,
+     size_t len,
+     const double *vals
+     )
+{
+    int iel;
+    unsigned char uc;
+    short ss;
+    int ii;
+    char gps[30];
+    float ff;
+    double dd;
+
+    if (len == 0)
+	return;
+    for (iel = 0; iel < len-1; iel++) {
+	switch (type) {
+	case NC_BYTE:
+	    uc = (unsigned char) vals[iel] & 0377;
+	    if (isprint(uc))
+		Printf ("'%c', ", uc);
+	    else
+		Printf ("'\\%o', ", uc);
+	    break;
+	case NC_SHORT:
+	    ss = vals[iel];
+	    Printf ("%ds, ", ss);
+	    break;
+	case NC_INT:
+	    ii = (int) vals[iel];
+	    Printf ("%d, ", ii);
+	    break;
+	case NC_FLOAT:
+	    ff = vals[iel];
+	    (void) sprintf(gps, float_att_fmt, ff);
+	    tztrim(gps);	/* trim trailing 0's after '.' */
+	    Printf ("%s, ", gps);
+	    break;
+	case NC_DOUBLE:
+	    dd = vals[iel];
+	    (void) sprintf(gps, double_att_fmt, dd);
+	    tztrim(gps);
+	    Printf ("%s, ", gps);
+	    break;
+	default:
+	    error("pr_att_vals: bad type");
+	}
+    }
     switch (type) {
-      case NC_BYTE:
-	gp.cp = (char *) vals;
-	for (iel = 0; iel < len; iel++)
-	{
-	  uc = *gp.cp++ & 0377;
-	  if (isprint(uc))
-	    Printf ("'%c'%s", uc, iel<len-1 ? ", " : "");
-	  else
-	    Printf ("'\\%o'%s", uc, iel<len-1 ? ", " : "");
-	}
+    case NC_BYTE:
+	uc = (unsigned char) vals[iel] & 0377;
+	if (isprint(uc))
+	    Printf ("'%c'", uc);
+	else
+	    Printf ("'\\%o'", uc);
 	break;
-      case NC_CHAR:
-	gp.cp = (char *) vals;
-	Printf ("\"");
-	/* adjust len so trailing nulls don't get printed */
-	sp = gp.cp + len - 1;
-	while (len > 0 && *sp-- == '\0')
-	    len--;
-	for (iel = 0; iel < len; iel++)
-	  switch (uc = *gp.cp++ & 0377) {
-	    case '\b':
-	      Printf ("\\b");
-	      break;
-	    case '\f':
-	      Printf ("\\f");
-	      break;
-	    case '\n':		/* generate linebreaks after new-lines */
-	      Printf ("\\n\",\n    \"");
-	      break;
-	    case '\r':
-	      Printf ("\\r");
-	      break;
-	    case '\t':
-	      Printf ("\\t");
-	      break;
-	    case '\v':
-	      Printf ("\\v");
-	      break;
-	    case '\\':
-	      Printf ("\\\\");
-	      break;
-	    case '\'':
-	      Printf ("\\'");
-	      break;
-	    case '\"':
-	      Printf ("\\\"");
-	      break;
-	    default:
-	      Printf ("%c",uc);
-	      break;
-	  }
-	Printf ("\"");
+    case NC_SHORT:
+	ss = vals[iel];
+	Printf ("%ds", ss);
 	break;
-      case NC_SHORT:
-	gp.sp = (short *) vals;
-	for (iel = 0; iel < len; iel++)
-	  Printf ("%ds%s",*gp.sp++,iel<len-1 ? ", " : "");
+    case NC_INT:
+	ii = (int) vals[iel];
+	Printf ("%d", ii);
 	break;
-      case NC_LONG:
-	gp.lp = (nclong *) vals;
-	for (iel = 0; iel < len; iel++)
-	  Printf ("%ld%s",*gp.lp++,iel<len-1 ? ", " : "");
+    case NC_FLOAT:
+	ff = vals[iel];
+	(void) sprintf(gps, float_att_fmt, ff);
+	tztrim(gps);
+	Printf ("%s", gps);
 	break;
-      case NC_FLOAT:
-	gp.fp = (float *) vals;
-	for (iel = 0; iel < len; iel++) {
-	    int ll;
-	    (void) sprintf(gps, float_att_fmt, * gp.fp++);
-	    tztrim(gps);	/* trim trailing 0's after '.' */
-	    Printf ("%s%s", gps, iel<len-1 ? ", " : "");
-	}
+    case NC_DOUBLE:
+	dd = vals[iel];
+	(void) sprintf(gps, double_att_fmt, dd);
+	tztrim(gps);
+	Printf ("%s", gps);
 	break;
-      case NC_DOUBLE:
-	gp.dp = (double *) vals;
-	for (iel = 0; iel < len; iel++) {
-	    (void) sprintf(gps, double_att_fmt, *gp.dp++);
-	    tztrim(gps);	/* trim trailing 0's after '.' */
-	    Printf ("%s%s", gps, iel<len-1 ? ", " : "");
-	}
-	break;
-      default:
+    default:
 	error("pr_att_vals: bad type");
     }
 }
 
 
 static void
-do_ncdump(path, specp)
-     char *path;
-     struct fspec* specp;
+pr_att(
+    int ncid,
+    int varid,
+    const char *varname,
+    int ia
+    )
+{
+    struct ncatt att;		/* attribute */
+	    
+    NC_CHECK( nc_inq_attname(ncid, varid, ia, att.name) );
+
+    Printf ("\t\t%s:%s = ", varname, att.name);
+
+    NC_CHECK( nc_inq_att(ncid, varid, att.name, &att.type, &att.len) );
+
+    if (att.len == 0) {	/* show 0-length attributes as empty strings */
+	att.type = NC_CHAR;
+	att.len = 1;
+    }
+    switch (att.type) {
+    case NC_CHAR:
+	att.string = (char *) malloc(att.len);
+	if (!att.string) {
+	    error("Out of memory!");
+	    NC_CHECK( nc_close(ncid) );
+	    return;
+	}
+	NC_CHECK( nc_get_att_text(ncid, varid, att.name, att.string ) );
+	pr_att_string(att.len, att.string);
+	free(att.string);
+	break;
+    default:
+	att.vals = (double *) malloc(att.len * sizeof(double));
+	if (!att.vals) {
+	    error("Out of memory!");
+	    NC_CHECK( nc_close(ncid) );
+	    return;
+	}
+	NC_CHECK( nc_get_att_double(ncid, varid, att.name, att.vals ) );
+	pr_att_vals(att.type, att.len, att.vals);
+	free(att.vals);
+	break;
+    }
+    Printf (" ;\n");
+}
+
+
+static void
+do_ncdump(const char *path, struct fspec* specp)
 {
     int ndims;			/* number of dimensions */
     int nvars;			/* number of variables */
@@ -265,20 +347,21 @@ do_ncdump(path, specp)
     int xdimid;			/* id of unlimited dimension */
     int dimid;			/* dimension id */
     int varid;			/* variable id */
-    struct ncdim dims[MAX_NC_DIMS]; /* dimensions */
-    long vdims[MAX_NC_DIMS];	/* dimension sizes for a single variable */
+    struct ncdim dims[NC_MAX_DIMS]; /* dimensions */
+    size_t vdims[NC_MAX_DIMS];	/* dimension sizes for a single variable */
     struct ncvar var;		/* variable */
     struct ncatt att;		/* attribute */
     int id;			/* dimension number per variable */
     int ia;			/* attribute number */
     int iv;			/* variable number */
     int is_coord;		/* true if variable is a coordinate variable */
-    int ncid = ncopen(path, NC_NOWRITE); /* netCDF id */
+    int ncid;			/* netCDF id */
     vnode* vlist = 0;		/* list for vars specified with -v option */
+    int nc_status;		/* return from netcdf calls */
 
-    if (ncid == -1) { 
-	error("ncopen failed on %s", path);
-	return;
+    nc_status = nc_open(path, NC_NOWRITE, &ncid);
+    if (nc_status != NC_NOERR) {
+	error("%s: %s", path, nc_strerror(nc_status));
     }
     /*
      * If any vars were specified with -v option, get list of associated
@@ -287,7 +370,7 @@ do_ncdump(path, specp)
     if (specp->nlvars > 0) {
 	vlist = newvlist();	/* list for vars specified with -v option */
 	for (iv=0; iv < specp->nlvars; iv++) {
-	    varid = ncvarid(ncid, specp->lvars[iv]);
+	    NC_CHECK( nc_inq_varid(ncid, specp->lvars[iv], &varid) );
 	    varadd(vlist, varid);
 	}
     }
@@ -302,26 +385,25 @@ do_ncdump(path, specp)
      * get number of dimensions, number of variables, number of global
      * atts, and dimension id of unlimited dimension, if any
      */
-    (void) ncinquire(ncid, &ndims, &nvars, &ngatts, &xdimid);
-
+    NC_CHECK( nc_inq(ncid, &ndims, &nvars, &ngatts, &xdimid) );
     /* get dimension info */
     if (ndims > 0)
       Printf ("dimensions:\n");
     for (dimid = 0; dimid < ndims; dimid++) {
-	(void) ncdiminq(ncid, dimid, dims[dimid].name, &dims[dimid].size);
+	NC_CHECK( nc_inq_dim(ncid, dimid, dims[dimid].name, &dims[dimid].size) );
 	if (dimid == xdimid)
-	  Printf ("\t%s = %s ; // (%d currently)\n",dims[dimid].name,
-		  "UNLIMITED", dims[dimid].size);
+	  Printf ("\t%s = %s ; // (%ld currently)\n",dims[dimid].name,
+		  "UNLIMITED", (long)dims[dimid].size);
 	else
-	  Printf ("\t%s = %ld ;\n",dims[dimid].name, dims[dimid].size);
+	  Printf ("\t%s = %ld ;\n", dims[dimid].name, (long)dims[dimid].size);
     }
 
     if (nvars > 0)
 	Printf ("variables:\n");
     /* get variable info, with variable attributes */
     for (varid = 0; varid < nvars; varid++) {
-	(void) ncvarinq(ncid, varid, var.name, &var.type, &var.ndims, var.dims,
-			&var.natts);
+	NC_CHECK( nc_inq_var(ncid, varid, var.name, &var.type, &var.ndims,
+			     var.dims, &var.natts) );
 	Printf ("\t%s %s", type_name(var.type), var.name);
 	if (var.ndims > 0)
 	  Printf ("(");
@@ -333,71 +415,28 @@ do_ncdump(path, specp)
 	Printf (" ;\n");
 
 	/* get variable attributes */
-	for (ia = 0; ia < var.natts; ia++) {
-	    (void) ncattname(ncid, varid, ia, att.name);
-	    
-	    Printf ("\t\t%s:%s = ", var.name, att.name);
-
-	    (void) ncattinq(ncid, varid, att.name, &att.type, &att.len);
-
-	    if (att.len == 0) {	/* show 0-length attributes as empty strings */
-		att.type = NC_CHAR;
-		att.val = (char *) malloc(1);
-		att.len = 1;
-		*(char *)att.val = '\0';
-	    } else {
-		att.val = (void *) malloc((unsigned)att.len*nctypelen(att.type));
-		if (!att.val) {
-		    error("Out of memory!");
-		    (void) ncclose(ncid);
-		    if (vlist)
-			free((char*)vlist);
-		    return;
-		}
-		(void) ncattget(ncid, varid, att.name, att.val);
-	    }
-	    pr_att_vals(att.type, att.len, att.val);
-	    Printf (" ;\n");
-	    free ((char *) att.val);
-	}
+	for (ia = 0; ia < var.natts; ia++)
+	    pr_att(ncid, varid, var.name, ia); /* print ia-th attribute */
     }
 
 
     /* get global attributes */
     if (ngatts > 0)
       Printf ("\n// global attributes:\n");
-    for (ia = 0; ia < ngatts; ia++) {
-	(void) ncattname(ncid, NC_GLOBAL, ia, att.name);
-	
-	Printf ("\t\t:%s = ", att.name);
-	
-	(void) ncattinq(ncid, NC_GLOBAL, att.name, &att.type, &att.len);
-	att.val = (void *) malloc((unsigned) (att.len*nctypelen(att.type)));
-	if (!att.val) {
-	    error("Out of memory!");
-	    (void) ncclose(ncid);
-	    if (vlist)
-		free(vlist);
-	    return;
-	}
-	(void) ncattget(ncid, NC_GLOBAL, att.name, att.val);
-
-	pr_att_vals(att.type, att.len, att.val);
-	Printf (" ;\n");
-	free ((char *) att.val);
-    }
+    for (ia = 0; ia < ngatts; ia++)
+	pr_att(ncid, NC_GLOBAL, "", ia); /* print ia-th global attribute */
     
     if (! specp->header_only) {
 	if (nvars > 0) {
-	    Printf ("\ndata:\n");
+	    Printf ("data:\n");
 	}
 	/* output variable data */
 	for (varid = 0; varid < nvars; varid++) {
 	    /* if var list specified, test for membership */
 	    if (specp->nlvars > 0 && ! varmember(vlist, varid))
 	      continue;
-	    (void) ncvarinq(ncid, varid, var.name, &var.type, &var.ndims,
-			    var.dims, &var.natts);
+	    NC_CHECK( nc_inq_var(ncid, varid, var.name, &var.type, &var.ndims,
+			    var.dims, &var.natts) );
 	    if (specp->coord_vals) {
 		/* Find out if this is a coordinate variable */
 		is_coord = 0;
@@ -419,7 +458,6 @@ do_ncdump(path, specp)
 	    if (var.ndims == 0
 		|| var.dims[0] != xdimid
 		|| dims[xdimid].size != 0) {
-		int errstate, ret;
 
 		/* Collect variable's dim sizes */
 		for (id = 0; id < var.ndims; id++)
@@ -427,12 +465,18 @@ do_ncdump(path, specp)
 		var.has_fillval = 1; /* by default, but turn off for bytes */
 
 		/* get _FillValue attribute */
-		errstate = ncopts;
-		ncopts = 0;	/* so no _FillValue won't result in error */
-		ret = ncattinq(ncid,varid,_FillValue,&att.type,&att.len);
-		ncopts = errstate;
-		if(ret != -1 && att.type == var.type && att.len == 1) {
-		    (void) ncattget(ncid, varid, _FillValue, &var.fillval);
+		nc_status = nc_inq_att(ncid,varid,_FillValue,&att.type,&att.len);
+		if(nc_status == NC_NOERR &&
+		   att.type == var.type && att.len == 1) {
+		    if(var.type == NC_CHAR) {
+			char fillc;
+			NC_CHECK( nc_get_att_text(ncid, varid, _FillValue,
+						  &fillc ) );
+			var.fillval = fillc;
+		    } else {
+			NC_CHECK( nc_get_att_double(ncid, varid, _FillValue,
+						    &var.fillval) );
+		    }
 		} else {
 		    switch (var.type) {
 		    case NC_BYTE:
@@ -440,19 +484,19 @@ do_ncdump(path, specp)
 			var.has_fillval = 0;
 			break;
 		    case NC_CHAR:
-			var.fillval.charv = FILL_CHAR;
+			var.fillval = NC_FILL_CHAR;
 			break;
 		    case NC_SHORT:
-			var.fillval.shortv = FILL_SHORT;
+			var.fillval = NC_FILL_SHORT;
 			break;
-		    case NC_LONG:
-			var.fillval.longv = FILL_LONG;
+		    case NC_INT:
+			var.fillval = NC_FILL_INT;
 			break;
 		    case NC_FLOAT:
-			var.fillval.floatv = FILL_FLOAT;
+			var.fillval = NC_FILL_FLOAT;
 			break;
 		    case NC_DOUBLE:
-			var.fillval.doublev = FILL_DOUBLE;
+			var.fillval = NC_FILL_DOUBLE;
 			break;
 		    default:
 			break;
@@ -460,7 +504,8 @@ do_ncdump(path, specp)
 		}
 		if (vardata(&var, vdims, ncid, varid, specp) == -1) {
 		    error("can't output data for variable %s", var.name);
-		    (void) ncclose(ncid);
+		    NC_CHECK(
+			nc_close(ncid) );
 		    if (vlist)
 			free(vlist);
 		    return;
@@ -470,16 +515,15 @@ do_ncdump(path, specp)
     }
     
     Printf ("}\n");
-    (void) ncclose(ncid);
+    NC_CHECK(
+	nc_close(ncid) );
     if (vlist)
 	free(vlist);
 }
 
 
 static void
-make_lvars(optarg, fspecp)
-     char *optarg;
-     struct fspec* fspecp;
+make_lvars(char *optarg, struct fspec* fspecp)
 {
     char *cp = optarg;
     int nvars = 1;
@@ -494,7 +538,6 @@ make_lvars(optarg, fspecp)
     fspecp->lvars = (char **) malloc(nvars * sizeof(char*));
     if (!fspecp->lvars) {
 	error("out of memory");
-	exit(EXIT_FAILURE);
     }
 
     cpp = fspecp->lvars;
@@ -506,7 +549,6 @@ make_lvars(optarg, fspecp)
 	*cpp = (char *) malloc(strlen(cp) + 1);
 	if (!*cpp) {
 	    error("out of memory");
-	    exit(EXIT_FAILURE);
 	}
 	strcpy(*cpp, cp);
 	cpp++;
@@ -516,33 +558,29 @@ make_lvars(optarg, fspecp)
 
 
 /*
- * Extract the significant-digits specifiers from the -d argument
- * (deprecated, use -p option instead) on the command-line and update the
- * default data formats appropriately.
+ * Extract the significant-digits specifiers from the -d argument on the
+ * command-line and update the default data formats appropriately.
  */
 static void
-set_sigdigs(optarg)
-     char *optarg;
+set_sigdigs(const char *optarg)
 {
-    char *ptr = optarg;
+    char *ptr1 = 0;
     char *ptr2 = 0;
-    long flt_digits = FLT_DIGITS;	/* default floating-point digits */
-    long dbl_digits = DBL_DIGITS;	/* default double-precision digits */
+    int flt_digits = FLT_DIGITS; /* default floating-point digits */
+    int dbl_digits = DBL_DIGITS; /* default double-precision digits */
 
     if (optarg != 0 && (int) strlen(optarg) > 0 && optarg[0] != ',')
-        flt_digits=strtol(optarg, &ptr, 10);
+        flt_digits = (int)strtol(optarg, &ptr1, 10);
 
     if (flt_digits < 1 || flt_digits > 20) {
 	error("unreasonable value for float significant digits: %d",
 	      flt_digits);
-	exit(EXIT_FAILURE);
     }
-    if (*ptr == ',')
-      dbl_digits = strtol(ptr+1, &ptr2, 10);
-    if (ptr2 == ptr+1 || dbl_digits < 1 || dbl_digits > 20) {
+    if (*ptr1 == ',')
+      dbl_digits = (int)strtol(ptr1+1, &ptr2, 10);
+    if (ptr2 == ptr1+1 || dbl_digits < 1 || dbl_digits > 20) {
 	error("unreasonable value for double significant digits: %d",
 	      dbl_digits);
-	exit(EXIT_FAILURE);
     }
     set_formats(flt_digits, dbl_digits);
 }
@@ -554,41 +592,36 @@ set_sigdigs(optarg)
  * and update the default data formats appropriately.
  */
 static void
-set_precision(optarg)
-     char *optarg;
+set_precision(const char *optarg)
 {
-    char *ptr = optarg;
+    char *ptr1 = 0;
     char *ptr2 = 0;
-    long flt_digits = FLT_DIGITS;	/* default floating-point digits */
-    long dbl_digits = DBL_DIGITS;	/* default double-precision digits */
+    int flt_digits = FLT_DIGITS;	/* default floating-point digits */
+    int dbl_digits = DBL_DIGITS;	/* default double-precision digits */
 
     if (optarg != 0 && (int) strlen(optarg) > 0 && optarg[0] != ',') {
-        flt_digits=strtol(optarg, &ptr, 10);
+        flt_digits = (int)strtol(optarg, &ptr1, 10);
 	float_precision_specified = 1;
     }
 
     if (flt_digits < 1 || flt_digits > 20) {
 	error("unreasonable value for float significant digits: %d",
 	      flt_digits);
-	exit(EXIT_FAILURE);
     }
-    if (*ptr == ',') {
-	dbl_digits = strtol(ptr+1, &ptr2, 10);
+    if (*ptr1 == ',') {
+	dbl_digits = (int) strtol(ptr1+1, &ptr2, 10);
 	double_precision_specified = 1;
     }
-    if (ptr2 == ptr+1 || dbl_digits < 1 || dbl_digits > 20) {
+    if (ptr2 == ptr1+1 || dbl_digits < 1 || dbl_digits > 20) {
 	error("unreasonable value for double significant digits: %d",
 	      dbl_digits);
-	exit(EXIT_FAILURE);
     }
     set_formats(flt_digits, dbl_digits);
 }
 
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char *argv[])
 {
     extern int optind;
     extern int opterr;
@@ -600,7 +633,7 @@ char *argv[];
 	  false,		/* just print coord vars? */
 	  false,		/* brief  comments in data section? */
 	  false,		/* full annotations in data section?  */
-	  LANG_NONE,		/* language conventions for indices */
+	  LANG_C,		/* language conventions for indices */
 	  0,			/* if -v specified, number of variables */
 	  0			/* if -v specified, list of variable names */
 	  };
@@ -637,7 +670,6 @@ char *argv[];
 	      break;
 	    default:
 	      error("invalid value for -b option: %s", optarg);
-	      exit(EXIT_FAILURE);
 	  }
 	  break;
 	case 'f':		/* full comments in data section */
@@ -651,14 +683,12 @@ char *argv[];
 	      break;
 	    default:
 	      error("invalid value for -f option: %s", optarg);
-	      exit(EXIT_FAILURE);
 	  }
 	  break;
 	case 'l':		/* maximum line length */
-	  max_len = strtol(optarg, 0, 0);
+	  max_len = (int) strtol(optarg, 0, 0);
 	  if (max_len < 10) {
 	      error("unreasonably small line length specified: %d", max_len);
-	      exit(EXIT_FAILURE);
 	  }
 	  break;
 	case 'v':		/* variable names */
